function p = completeParams(p)
% completeParams: adds missing default parameter values 
%
% Each root type with index 'i' is described by the following parameters:
% p{i}.lb = [mean, std]     length of the basal zone (default = [0,0])
% p{i}.la = [mean, std]     length of the apical zone (default = [10,0])
% p{i}.ln = [mean, std]     inter-branch distance (default = [0,0])
% p{i}.nob = [mean, std]    number of branches (default = [0,0])
% p{i}.r = [mean, std]      initial growth speed (cm/day) (default = [1,0])
% p{i}.a = [mean, std]      root radius (constant along axis) (default =
%                           [1e-2,0]) 
% p{i}.color = [r,g,b]      color of this root type (default =
%                           [150/255,150/255,50/255]) 
% p{i}.tropism = [t1,a1,t2,a2,..., N, s,]  tI is the type (0=Plagio-,
%                           1=Gravi-, 2=Exo-, 3=Chemotropism), aI is the
%                           factor of the linear combination, N is the
%                           number of tries, s is the maximal angular
%                           deviation in root heading (default=[1 1 pi/20])                                                                                                 
% p{i}.dx                   axial resolution (default = 0.1)
% p{i}.successor = [I,P]    I indices of successive types, P probability
%                           (default = [])            
% p{i}.theta = [mean, std]  angle between this type and the predecessor
%                           (default [70/180*pi,0]) 
% p{i}.rlt = [mean, std]    root life time  (default = [inf,0]
% p{i}.geometry = @(p)      signed distance function of boundaries (e.g.
%                           plant pot) (default = []).
% p{i}.lambda = @(t,y,r,k)  growth function lambda returns length at age t,
%                           where y gives position and heading of root tip, 
%                           r is the initial growth speed and k the final
%                           root length (default = negative exponential)
% p{i}.tau = @(l,r,k)       approximated inverse of lambda (default =
%                           inverse of negative exponential) 
% p{i}.name = 'name'        name of the root type (default='unknown')
% 
% Call this function before passing the parameter list to ruleCreateRoot.
%
% Parameters:
% p                         a list of parameter sets (for each root type)
%
%
% See also: ruleCreateRoot, createRootSystem.
%
% Copyright 2012 Daniel Leitner. See license.txt for details.
%

%
% default parameters with std = 0
%
defaultoptions=struct('lb',0,'la',10,'ln',0,'nob',0,'r',1,'a',0.01,...
    'color',[150/255,150/255,50/255],'tropism',[1 1 pi/20], 'dx', 0.1, ...
    'successor',[],'theta',70/180*pi,'rlt',inf,'geometry',[],...
    'lambda',@(t,y,r,k) k*(1-exp(-(r/k)*t)),...
    'tau', @(l,r,k) -k/r*log((k-l)/k),...
    'name','unknown');

%
% add missing parameters to all root types
%
for i = 1 : length(p)
        
    tags = fieldnames(defaultoptions);
    for j=1:length(tags)
        if(~isfield(p{i},tags{j}))
            p{i}.(tags{j})=defaultoptions.(tags{j}); % add default
        end
        if isreal(p{i}.(tags{j})) && length(p{i}.(tags{j}))==1 && ...
                ~strcmp(tags{j},'successor');
            p{i}.(tags{j})=[p{i}.(tags{j}),0]; % add zero std
        end
    end
    
    l=length(p{i}.tropism);
    if  (l~=3) && (mod(l,2)~=0)
        warning('completeParams:wrongargs','wrong number of tropism arguments');
    end
    
    if(length(tags)~=length(fieldnames(p{i})))
        warning('completeParams:unknownoption','unknown options found');
    end
    
    p{i}.color = [p{i}.color, i]; % add index
    
end

