function [S, nodes, indS] = getGraph(str, initial)
% getGraph: Returns a graph representation of the root system
%
% This function is esssential for computations within the root system.
%
% Parameters:
% str           :list of l-system letters
% (initial)     :initial values of the Turtle. Default: initial.xT=[0,0,0]
%       initial.wT=0.1, initial.cT = [0,1,0], R=diag([1,1,1])
%
% S             :connectivity matrix, entries are the edges numbers
% nodes         :graph node points
% indS          :list of segment indices corresponding to the edges
%
% See also: getSegments, getPolylines.
%
% Copyright 2012 Daniel Leitner. See license.txt for details.
%

if (nargin<2)
    xT = [ 0; 0; 0];
    R = diag([1 1 1]);
else
    xT = initial.xT;
    R = initial.R;
end
Rbk = [ cos(pi) sin(pi) 0; -sin(pi) cos(pi) 0; 0 0 1];


sC = 1; % segment index (within edge)
nC = 1; % node index
eC = 1; % edge index
c = 1; % segment counter
stkPtr = 1; % stack pointer

indices(eC,1:2) = [0,0]; % edge list
indS{eC} = []; % segment index list per edge
nodes(nC,:) = xT; % nodes

sni = 1; % start node index

for i=1:length(str)
    
    switch str{i}.name
        case '#'
        case 'C'
        case 'f'
            lenF = str{i}.length;
            xT = xT + lenF*R(:,1);
        case 'F'
            lenF = str{i}.length;
            xT = xT + lenF*R(:,1);
            indS{eC}(sC) = c;
            sC = sC + 1;
            c = c+1; % segment index
        case '+'
            a = str{i}.delta;
            R = R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case '-'
            a = str{i}.delta;
            R = R * [ cos(-a) sin(-a) 0; -sin(-a) cos(-a) 0; 0 0 1 ];
        case '&'
            a = str{i}.delta;
            R = R * [ cos(a) 0  -sin(a) ; 0 1 0; sin(a) 0 cos(a) ];
        case '^'
            a = str{i}.delta;
            R = R * [ cos(-a) 0 -sin(-a) ; 0 1 0; sin(-a) 0 cos(-a)];
        case '\'
            a = str{i}.delta;
            R = R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
        case '/'
            a = str{i}.delta;
            R = R * [ 1 0 0; 0 cos(-a) -sin(-a) ; 0 sin(-a) cos(-a) ];
        case '|'
            R = R * Rbk;
        case '[' %push the stack
            stack(stkPtr).xT = xT ;
            stack(stkPtr).R = R;
             
            if sC>1  % if there are segments in this edge
                nC=nC+1;
                nodes(nC,:) = xT;
                indices(eC,1:2) = [sni, nC];
                sni = nC;
                eC=eC+1;
                sC=1;
            end
 
            stack(stkPtr).sni = sni;           
            stkPtr = stkPtr +1 ;   
            
        case ']' %pop the stack
                       
            if sC>1  % if there are segments in this edge
                nC=nC+1;
                nodes(nC,:) = xT;
                indices(eC,1:2) = [sni, nC];
                eC=eC+1;
                sC=1;
            end
            
            stkPtr = stkPtr -1;            
            sni = stack(stkPtr).sni;
            xT = stack(stkPtr).xT ;
            R = stack(stkPtr).R;            
    end
    
end

S = sparse(indices(:,1),indices(:,2), 1:eC-1, size(nodes,1),size(nodes,1));


