function [lines,diameters,colors,times,types,indS,indE] = getPolylines(str, initial)
% getPolyline: Interprets a L-System string graphically using Turtle graphic.
%
% The function returns a list of polylines, diameters, colors and segment
% creation times. Every poly line represents a single root axis. The result
% can be represented using plotTubes.
%
% Parameters:
% str             :List of L-System letters
% (initial)       :Initial values of the Turtle. Default: initial.xT=0,
%       initial.yT=0, initial.zT=0, initial.wT=0.1, initial.cT = [0,1,0],
%       R=diag([1,1,1])
%
% lines           :a list of vectors of points, {1:nor}(1:nov(r))
%       nor: number of roots, nov: number of vertices per root
% diameters       :a vector of diameters, (1:nor)
% colors          :a vector of stream colors, {1:nor}(1:nov(r)-1,3)
% times           :a list of segment creation times, {1:nor}(1:nov(r)-1)
% types           :a vector of stream root type indices (1:nor)
% indS            :a list of segment indices, {1:nor}(1:nov(r)-1)
% indE            :a list of edge indices, {1:nor}(1:noe(r))
%
% See also: plotTubes, getSegments, getGraph.
%
% Copyright 2008-2012 Daniel Leitner. See license.txt for details.
%

if (nargin<2)  
    xT = [ 0; 0; 0];
    dT = 0.1;
    cT = [ 0, 1, 0, 0];
    R = diag([1 1 1]);
else
    xT = initial.xT;   
    dT = initial.dT;
    cT = initial.cT;
    R = initial.R;
end
Rbk = [ cos(pi) sin(pi) 0; -sin(pi) cos(pi) 0; 0 0 1];

vC = 1; % vertex counter
stC = 1; % stream counter
nofST = 1; % total number of streams
c = 1; % segment counter
stkPtr = 1; % stack pointer

lines{stC}(vC,:) = xT;
times{stC}(vC) = 0;
diameters(stC) = dT;
colors{stC}(vC,1:3)  = cT(1:3);
types(stC) = cT(4);

% a link to getSegments
indS{stC}(vC) = 0;

% a link to getGraph
eC = 1; % edge counter
sC = 1; % edge segment counter
indE{stC} = [];

vC = vC + 1;

for i=1:length(str)
    
    switch str{i}.name
        case '#'
            dT = str{i}.width;
            diameters(stC) = dT;
        case 'C'
            cT = str{i}.color;
            types(stC)=cT(4); 
        case 'f'
            lenF = str{i}.length;
            xT = xT + lenF*R(:,1);   
        case 'F'
            lenF = str{i}.length;
            t = str{i}.time;
            xT = xT + lenF*R(:,1);
            lines{stC}(vC,:) = xT;
            times{stC}(vC-1) = t;
            indS{stC}(vC-1) = c;
            colors{stC}(vC-1,1:3) =cT(1:3);
            vC = vC + 1;
            sC = sC + 1;
            c = c + 1; 
        case '+'
            a = str{i}.delta;
            R = R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case '-'
            a = str{i}.delta;
            R = R * [ cos(-a) sin(-a) 0; -sin(-a) cos(-a) 0; 0 0 1 ];
        case '&'
            a = str{i}.delta;
            R = R * [ cos(a) 0  -sin(a) ; 0 1 0; sin(a) 0 cos(a) ];
        case '^'
            a = str{i}.delta;
            R = R * [ cos(-a) 0 -sin(-a) ; 0 1 0; sin(-a) 0 cos(-a)];
        case '\'
            a = str{i}.delta;
            R = R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
        case '/'
            a = str{i}.delta;
            R = R * [ 1 0 0; 0 cos(-a) -sin(-a) ; 0 sin(-a) cos(-a) ];
        case '|'
            R = R * Rbk;
        case '[' %push the stack
            
            if sC>1  % if there are segments in this edge
                indE{stC}(end+1)=eC;
                eC=eC+1;
                sC=1;                
            end            
            
            stack(stkPtr).xT = xT ;
            stack(stkPtr).dT = dT;
            stack(stkPtr).cT = cT;
            stack(stkPtr).R = R;
            stack(stkPtr).vC = vC;
            stack(stkPtr).stC = stC;
            stkPtr = stkPtr +1 ; 
            
            stC  = nofST + 1; % start new stream            
            nofST = nofST +1;
            vC = 1;
            lines{stC}(vC,:) = xT;   
            times{stC} = [];
            indS{stC} = [];
            colors{stC} = [];
            indE{stC} = [];
            vC = vC +1;             
            
        case ']' %pop the stack   
            
            diameters(stC) = dT;            
            types(stC) = cT(4);            
                        
            if sC>1  % if there are segments in this edge
                indE{stC}(end+1)=eC;
                eC=eC+1;
                sC=1;                
            end
            
            stkPtr = stkPtr - 1;
            xT = stack(stkPtr).xT ;
            dT = stack(stkPtr).dT;
            cT = stack(stkPtr).cT;
            R = stack(stkPtr).R;
            vC= stack(stkPtr).vC;
            stC = stack(stkPtr).stC;
    end
    
end



