function [x1,x2,r,color,time,type,indS]=getSegments(str, initial)
% getSegments: Gets the segments, radii and other attributes of a root system
%
% Parameters:
% str           :list of l-system letters
% (initial)     :initial values of the Turtle. Default: initial.xT=[0,0,0]
%       initial.wT=0.1, initial.cT = [0,1,0], R=diag([1,1,1])
%       
% x1            :start points of the segments
% x2            :end points of the segments
% r             :radii of the segments
% color         :color of the segment
% time          :creation times of the segment
% type          :index of root system type
% indS          :index within the l-system string
%
% See also: plotLines, getPolylines, getGraph.
%
% Copyright 2008-2012 Daniel Leitner. See license.txt for details.
%

if (nargin<2)
    xT = [ 0; 0; 0];
    dT = 0.1;
    cT = [0, 1, 0, 0];
    R = diag([1 1 1]);
else
    xT = initial.xT;
    dT = initial.dT;
    cT = initial.cT;
    R = initial.R;
end
Rbk = [ cos(pi) sin(pi) 0; -sin(pi) cos(pi) 0; 0 0 1];

c = 1; % segment counter
stkPtr = 1; % stack pointer

N = getTotal(str,'f');
x1 = zeros(N,3);
x2 = zeros(N,3);
r = zeros(N,1);
color = zeros(N,3);
time = zeros(N,1);
type = zeros(N,1);
indS = zeros(N,1);

for i=1:length(str)

    switch str{i}.name
        case '#'
            dT = str{i}.width;
        case 'C'
            cT = str{i}.color;
        case 'f'
            lenF = str{i}.length;
            xT = xT + lenF*R(:,1);    
        case 'F'
            lenF = str{i}.length;
            t = str{i}.time; % each F has a creation time
            time(c) = t;
            indS(c) = i;
            newxT = xT + lenF*R(:,1);
            x1(c,:) = xT;
            x2(c,:) = newxT;
            r(c) = dT/2;
            color(c,:)= cT(1:3);
            type(c) = cT(4); % the type is hidden in color
            xT = newxT;
            c=c+1;
        case '+'
            a = str{i}.delta;
            R = R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case '-'
            a = str{i}.delta;
            R = R * [ cos(-a) sin(-a) 0; -sin(-a) cos(-a) 0; 0 0 1 ];
        case '&'
            a = str{i}.delta;
            R = R * [ cos(a) 0  -sin(a) ; 0 1 0; sin(a) 0 cos(a) ];
        case '^'
            a = str{i}.delta;
            R = R * [ cos(-a) 0 -sin(-a) ; 0 1 0; sin(-a) 0 cos(-a)];
        case '\'
            a = str{i}.delta;
            R = R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
        case '/'
            a = str{i}.delta;
            R = R * [ 1 0 0; 0 cos(-a) -sin(-a) ; 0 sin(-a) cos(-a) ];
        case '|'
            R = R * Rbk;
        case '[' %push the stack
            stack(stkPtr).xT = xT ;
            stack(stkPtr).dT = dT;
            stack(stkPtr).cT = cT;
            stack(stkPtr).R = R;
            stkPtr = stkPtr +1 ;
        case ']' %pop the stack
            stkPtr = stkPtr -1;
            xT = stack(stkPtr).xT ;
            dT = stack(stkPtr).dT;
            cT = stack(stkPtr).cT;
            R = stack(stkPtr).R;
    end
    
end

