function r = getTurtle()
% getTurtle: Sets up basic L-System letters containing turtle commands.
%
% The following letters are interpreted by 3D turtle:
% field         : name  :  description
% --------------------------------------
% F             :  'F'  : Forward (length)
% Color         :  'C'  : Color (color)
% Diameter      :  '#'  : Line width (width)
% Push          :  '['  : Push turtle state
% Pop           :  ']'  : Pop turtle state
% TurnL         :  '+'  : Turn left by angle delta
% TurnR         :  '-'  : Turn right by angle delta
% PitchD        :  '&'  : Pitch down by angle delta
% PitchU        :  '^'  : Pitch up by angle delta
% RollL         :  '\'  : Roll left by angle delta
% RollR         :  '/'  : Roll right by angle delta.
% TurnAround    :  '|'   : Turn around
% BranchRotate  :  '3'  : Randomly roll and turn left by angle delta (*)
% Bend3D        :  'b'  : rule for root tip deflection (*)
%
% (*) this commands are not interpreted by the turtle but replaced by basic
%     turtle commands.
%
% Parameters:
% r        :A structure containing the basic L-System letters.
%
% See also: plotTubes, plotLines
%
% Copyright 2008-2012 Daniel Leitner. See license.txt for details.
%

global rules;

id = @(x,dt) {x};

rules.F.name = 'F';
rules.F.length = 1;
rules.F.apply = id;
rules.F.time = 0;

rules.Color.name = 'C';
rules.Color.color = [.5 .4 0];
rules.Color.apply = id;

rules.Diameter.name = '#';
rules.Diameter.width = 1;
rules.Diameter.apply = id;

rules.Push.name = '[';
rules.Push.apply = id;

rules.Pop.name = ']';
rules.Pop.apply = id;

rules.TurnL.name='+';
rules.TurnL.delta = pi/3;
rules.TurnL.apply = id;

rules.TurnR.name='-';
rules.TurnR.delta = pi/3;
rules.TurnR.apply = id;

rules.PitchD.name='&';
rules.PitchD.delta = pi/3;
rules.PitchD.apply = id;

rules.PitchU.name='^';
rules.PitchU.delta = pi/3;
rules.PitchU.apply = id;

rules.RollL.name='\';
rules.RollL.delta = pi/3;
rules.RollL.apply = id;

rules.RollR.name='/';
rules.RollR.delta = pi/3;
rules.RollR.apply = id;

rules.TurnAround.name='|';
rules.TurnAround.apply = id;

rules.BranchRotate.name = '3';
rules.BranchRotate.delta = pi/3;
rules.BranchRotate.sigma = 0;
rules.BranchRotate.apply = @(x,dt) ruleBranchRotate(x);

rules.Bend3D.name = 'b';
rules.Bend3D.sigma = pi/30;
rules.Bend3D.apply = @(x,y,dx) ruleBend3D(x,y);

r=rules;

function new = ruleBranchRotate(x)
global rules;
r = rules.RollL;
r.delta = 2*pi*rand;
t = rules.TurnL;
t.delta = x.delta;
new = {r,t};


function new = ruleBend3D(x)
global rules;
r = rules.RollL;
if (x.sigma==0)
    r.delta =0 ;
else
    r.delta= rand*2*pi;
end
t = rules.TurnL;
t.delta = rand*x.sigma;
new = {r,t};

