function new = ruleCreateRoot(x,p,i,dt)
% ruleCreateRoot: Production rule that creates a root of type i.
%
% Parameters:
% x         :letter that is replaced by the production rule
% p         :list of root system parameters (see completeParams)
% i         :index of the root type to be created
% dt        :time step
%
% new       :resulting l-system string
%
% See also: completeParams.
%
% Copyright 2008-2012 Daniel Leitner. See license.txt for details.
%

global rules;

%%
% apply standard deviation
%
la = max(p{i}.la(1) + randn*p{i}.la(2),0); % length of apical zone
lb = max(p{i}.lb(1) + randn*p{i}.lb(2),0); % length of basal zone
nob = max(p{i}.nob(1) + round(randn*p{i}.nob(2)),0); % number of branches
ln = max(p{i}.ln(1) + randn(nob,1)*p{i}.ln(2),0); % inter-branch distance
a = max(p{i}.a(1) + randn*p{i}.a(2),0); % root radius
theta = max(p{i}.theta(1) + randn*p{i}.theta(2),0); % angle between successive branches
color=p{i}.color; % color of the root
r = max(p{i}.r(1) + randn*p{i}.r(2),0); % initial growth speed
rlt=max(p{i}.rlt(1)+randn*p{i}.rlt(2),0);  %root life time

k = la+lb+sum(ln); % total length of fully grown root
lambda = @(t,y) p{i}.lambda(t,y,r,k); % redefine lambda with current r and k
tau = @(l) p{i}.tau (l,r,k); % inverse of lambda

dx = p{i}.dx(1); % resolution along root axis


%%
% if the root has a certain life span
%
die = {};
if (rlt~=inf)
    cdie = rules.Color;
    cdie.color = [0,0,0,p{i}.color(4)];
    die.name='d';
    die.apply = @(x,dt) ruleDelay(x,rlt,cdie,dt);
    die.t=0;
    die.xT = x.xT; % copy labels
    die.dT = x.dT;
    die.cT = x.cT;
    die.tT = x.tT;
    die.R = x.R;
end


%%
% initial string
rules.Diameter.width=2*a;
rules.BranchRotate.delta = theta;
rules.Color.color=color;
br = rules.BranchRotate.apply(rules.BranchRotate);
initial = [{rules.Push}, {rules.Diameter}, {rules.Color}, die, br];


%%
% handle tropism 
%
N = p{i}.tropism(end-1);
sigma = p{i}.tropism(end);
trop = p{i}.tropism(1:end-2);

if length(trop) == 1
    trop(1,2) = 1; % add default linear factor
end

K = length(trop)/2;
objfun = cell(K,1);
lf =  zeros(K,1);
for j = 1:K
    lf(j) = trop(j*2); % linear factor
    type = trop(j*2-1);    
    switch(type) % 0 plagio, 1 gravi, 2 exo, 3 chemo
        case 0
            objfun{j} = getTropismObjective(0);
        case 1
            objfun{j} = getTropismObjective(1);
        case 2
            x_ = x;
            x_ = labelTip(x_, initial, 0); % find direction for exotropism
            objfun{j} = getTropismObjective(2,x_.R);
        case 3
            objfun{j} = getTropismObjective(3,dx);
    end
end

switch K % linearly combine up to four tropisms
    case 1
        tropism = @(y,dx) ruleTropismN(y,N,sigma,dx,objfun{1});        
    case 2
        fof = @(b,a,y) lf(1)*objfun{1}(b,a,y) + lf(2)*objfun{2}(b,a,y);
        tropism = @(y,dx) ruleTropismN(y,N,sigma,dx,fof);
    case 3
        fof = @(b,a,y) lf(1)*objfun{1}(b,a,y) + lf(2)*objfun{2}(b,a,y) + ...
            lf(3)*objfun{3}(b,a,y); 
        tropism = @(y,dx) ruleTropismN(y,N,sigma,dx,fof);        
    case 4
        fof = @(b,a,y) lf(1)*objfun{1}(b,a,y) + lf(2)*objfun{2}(b,a,y) + ...
            lf(3)*objfun{3}(b,a,y) + lf(4)*objfun{4}(b,a,y); 
        tropism = @(y,dx) ruleTropismN(y,N,sigma,dx,fof);          
end

if ~isempty(p{i}.geometry) 
    tropism = @(y,dx) ruleRestrictTropism(y,dx,p{i}.geometry,tropism);
end


%%
% create root
%
if r>0 && k>0 && rlt>0
    
    if exist('rules','var')~=1 % if the global variable is not set define it
        rules = getTurtle();
    end
    
    % apical zone
    if la>dx/10
        apical.name = 'G';
        apical.t = 0;
        apical.l = 0;
        t0 = tau(k-la);
        tend = tau(k-dx/10);
        apical.apply = @(x,dt) ruleSectionGrowth(x,lambda,lambda(t0,apical),...
            lambda(tend,apical),t0, tend, rules.TurnAround,tropism,rlt,dx,dt);
    else
        apical = rules.TurnAround;
    end
    
    % branching zone
    if (nob>0)
        branching.name='G';
        branching.c=0;        
        l = lb + [0; cumsum(ln)];
        times = tau(l);
        delays = tau(l+la) - tau(l);
        delays(end)=[];         
        branching.apply = @(x,dt) ruleBranch(x,lambda,times,delays,...
            p{i}.successor,apical,tropism,rlt,dx,dt,p);
    else
        branching=apical;
    end
    
    % basal zone
    if lb>0
        basal.name = 'G';
        basal.t = 0;
        basal.l = 0;
        t0=0;
        tend = tau(lb);
        basal.apply = @(x,dt) ruleSectionGrowth(x,lambda,lambda(t0,basal),...
            lambda(tend,basal),t0, tend, branching,tropism,rlt,dx,dt);
    else
        basal=branching;
    end
    
    basal.xT = x.xT; % copy labels
    basal.dT = x.dT;
    basal.cT = x.cT;
    basal.tT = x.tT;
    basal.R = x.R;
             
    % start root 
    basal = labelTip(basal, initial, 0);    
    branch = basal.apply(basal,dt);   %apply time overhead
    new = [initial, branch, {rules.Pop}];
    
else    
    new ={};    
    
end
